@@ -1,28 +1,23 @@
 ##' @export
-`%++%.lvm` <- function(x,y) merge(x,y)
+`%++%.lvm` <- function(x, y) merge(x, y)
 
 ##' @export
-"+.lvm" <- function(x,...) {
-  merge(x,...)
+"+.lvm" <- function(x, ...) {
+  merge(x, ...)
 }
 
-## ##' @export
-## "+.lm" <- function(x,...) {
-##   merge(x,...)
-## }
-
 ##' @export
-merge.lvm <- function(x,y,...) {
-  objects <- list(x,y,...)
+merge.lvm <- function(x, y, ...) {
+  objects <- list(x, y, ...)
   if (length(objects)<2) return(x)
   m <- objects[[1]]
-  for (i in seq(2,length(objects))) {
+  for (i in seq(2, length(objects))) {
     m2 <- objects[[i]]
     if (length(latent(m2))>0)
       latent(m) <- latent(m2)
     if (length(m2$constrain)>0)
       m$constrain <- c(m$constrain,m2$constrain)
-    M <- (index(m2)$A)
+    M <- (index(m2)$M)
     P <- (index(m2)$P)
     nn <- vars(m2)
     for (j in seq_len(nrow(M))) {
@@ -34,6 +29,7 @@ merge.lvm <- function(x,y,...) {
           val[idx.] <- m2$fix[j,idx][idx.]
         regression(m,to=nn[idx],from=nn[j],messages=0) <- val
       }
+
       P0 <- P[j,]; P0[seq_len(j-1)] <- 0
       idx <- P[j,]!=0 | m2$covfix[j,]==0
       idx[is.na(idx)] <- FALSE
@@ -57,39 +53,76 @@ merge.lvm <- function(x,y,...) {
 
 
 ##' @export
-"+.estimate" <- function(x,...) {
-  merge(x,...)
+"+.estimate" <- function(x, ...) {
+  merge(x, ...)
 }
 
 ##' @export
-merge.estimate <- function(x,y,...,id,paired=FALSE,labels=NULL,keep=NULL,subset=NULL) {
-    objects <- list(x,y, ...)
+"-.estimate" <- function(x,...) {
+  res <- merge(x, ...)
+  estimate(res, pairwise.diff(length(coef(res))))
+}
+
+##' @export
+merge.estimate <- function(x,y,...,
+                           id,
+                           paired=FALSE,
+                           labels=NULL,
+                           keep=NULL,
+                           subset=NULL,
+                           regex=FALSE,
+                           ignore.case=FALSE) {
+    objects <- list(x, estimate(y), ...)
     if (length(nai <- names(objects)=="NA")>0)
     names(objects)[which(nai)] <- ""
     if (!missing(subset)) {
-        coefs <- unlist(lapply(objects, function(x) coef(x)[subset]))
+      if (regex) {
+
+      }
+      coefs <- unlist(lapply(objects, function(x) coef(x)[subset]))
     } else {
-        coefs <- unlist(lapply(objects,coef))
+      coefs <- unlist(lapply(objects,coef))
     }
     if (!is.null(labels)) {
-        names(coefs) <- labels
+      names(coefs) <- labels
     } else {
-        names(coefs) <- make.unique(names(coefs))
+      names(coefs) <- make.unique(names(coefs))
+    }
+    if (regex) {
+      if (!is.null(keep)) {
+        cc <- names(coefs)
+        keep <- unlist(lapply(keep, function(x) {
+          cc[grepl(x, cc, perl = TRUE, ignore.case=ignore.case)]
+        }))
+      }
+    }
+    if (any(unlist(lapply(objects, function(x) is.null(IC(x)))))) {
+      ## No iid decomposition/influence functions
+      V <- lapply(objects, vcov)
+      V <- Reduce(function(...) blockdiag(..., pad=NA), V)
+      return(estimate(coef=coefs, vcov=V, keep=keep, ...))
     }
     if (!missing(id) && is.null(id)) { ## Independence between datasets in x,y,...
-        nn <- unlist(lapply(objects,function(x) nrow(x$iid)))
-        cnn <- c(0,cumsum(nn))
+        nn <- unlist(lapply(
+          objects,
+          function(x) nrow(x$IC)
+        ))
+        cnn <- c(0, cumsum(nn))
         id <- list()
-        for (i in seq_along(nn)) id <- c(id,list(seq(nn[i])+cnn[i]))
+        for (i in seq_along(nn)) {
+          id <- c(id, list(seq(nn[i]) + cnn[i]))
+        }
     }
     if (missing(id)) {
-        if (paired) { ## One-to-one dependence between observations in x,y,...
-            id <- rep(list(seq(nrow(x$iid))),length(objects))
+      if (paired) { ## One-to-one dependence between observations in x,y,...
+        id <- lapply(objects, function(x) {
+          seq_len(NROW(x$IC))
+        })
         } else {
-            id <- lapply(objects,function(x) x$id)
+            id <- lapply(objects, function(x) x$id)
         }
     } else {
-        nn <- unlist(lapply(objects,function(x) NROW(iid(x))))
+        nn <- unlist(lapply(objects,function(x) NROW(IC(x))))
         if (length(id)==1 && is.logical(id)) {
             if (id) {
                 if (any(nn[1]!=nn)) stop("Expected objects of the same size: ", paste(nn,collapse=","))
@@ -105,45 +138,62 @@ merge.estimate <- function(x,y,...,id,paired=FALSE,labels=NULL,keep=NULL,subset=
         idlen <- unlist(lapply(id,length))
         if (!identical(idlen,nn)) stop("Wrong lengths of 'id': ", paste(idlen,collapse=","), "; ", paste(nn,collapse=","))
     }
-    ##if (any(unlist(lapply(id,is.null)))) stop("Id needed for each model object")
-    ##iid <- Reduce("cbind",lapply(objects,iid))
-    ids <- iidall <- c(); count <- 0
+    ids <- ic_all <- c(); count <- 0
     for (z in objects) {
         count <- count+1
         clidx <- NULL
         id0 <- id[[count]]
-        iidz <- iid(z)
+        icz <- IC(z)
         if (is.null(id0)) {
-            id0 <- rownames(iidz)
+            id0 <- rownames(icz)
             if (is.null(id0)) stop("Need id for object number ", count)
         }
-        if (!missing(subset)) iidz <- iidz[,subset,drop=FALSE]
+        if (!missing(subset)) icz <- icz[,subset,drop=FALSE]
         if (!lava.options()$cluster.index) {
-            iid0 <- matrix(unlist(by(iidz,id0,colSums)),byrow=TRUE,ncol=ncol(iidz))
+            ic0 <- matrix(unlist(by(icz,id0,colSums)),byrow=TRUE,ncol=ncol(icz))
             ids <- c(ids, list(sort(unique(id0))))
-
         } else {
             if (!requireNamespace("mets",quietly=TRUE)) stop("'mets' package required")
-            clidx <- mets::cluster.index(id0,mat=iidz,return.all=TRUE)
-            iid0 <- clidx$X
+            clidx <- mets::cluster.index(id0,mat=icz,return.all=TRUE)
+            ic0 <- clidx$X
             ids <- c(ids, list(id0[as.vector(clidx$firstclustid)+1]))
         }
-        iidall <- c(iidall, list(iid0))
+        ic0 <- ic0*NROW(ic0)/length(id0)
+        ic_all <- c(ic_all, list(ic0))
     }
     id <- unique(unlist(ids))
-    iid0 <- matrix(0,nrow=length(id),ncol=length(coefs))
+    ic0 <- matrix(NA, nrow=length(id),ncol=length(coefs))
     model.index <- c()
     colpos <- 0
     for (i in seq(length(objects))) {
         relpos <- seq_along(coef(objects[[i]]))        
         if (!missing(subset)) relpos <- seq_along(subset)
-        iid0[match(ids[[i]],id),relpos+colpos] <- iidall[[i]]
-        model.index <- c(model.index,list(relpos+colpos))
+        ic0[match(ids[[i]], id), relpos + colpos] <- ic_all[[i]]
+        midx <- objects[[i]]$model.index
+        if (!is.null(midx)) {
+          midx <- lapply(midx, function(x) {
+            intersect(x, relpos) + colpos
+          })
+        } else {
+          midx <- list(relpos + colpos)
+        }
+        model.index <- c(model.index, midx)
         colpos <- colpos+tail(relpos,1)
     }
-    rownames(iid0) <- id
-    res <- estimate.default(NULL, coef=coefs, stack=FALSE, data=NULL, iid=iid0, id=id, keep=keep)
-    res$model.index <- model.index
+    rownames(ic0) <- id
+    ## Rescale each column according to I(obs)/pr(obs)
+    for (i in seq(NCOL(ic0))) {
+      pr <- mean(!is.na(ic0[,i]))
+      ic0[,i] <- ic0[,i]/pr
+    }
+    ic0[is.na(ic0)] <- 0
+
+    res <- estimate.default(NULL,
+      coef = coefs, stack = FALSE, data = NULL,
+      IC = ic0, id = id, keep = keep
+      )
+    if (is.null(keep))
+      res$model.index <- model.index
     return(res)
 }
 
